
/******************************************************************************/
/*                                                                            */
/* Broadcom BCM4400 Linux Network Driver, Copyright (c) 2002 Broadcom         */
/* Corporation.                                                               */
/* All rights reserved.                                                       */
/*                                                                            */
/* This program is free software; you can redistribute it and/or modify       */
/* it under the terms of the GNU General Public License as published by       */
/* the Free Software Foundation, located in the file LICENSE.                 */
/*                                                                            */
/* History:                                                                   */
/*    02/25/00 Hav Khauv        Initial version.                              */
/******************************************************************************/

#ifndef B44LM_H
#define B44LM_H

#include "b44queue.h"



/******************************************************************************/
/* Basic types. */
/******************************************************************************/

#define TRUE           1
#define FALSE          0

#ifndef OFFSETOF
#define OFFSETOF(_s, _m)    (B44_MM_PTR(&(((_s *) 0)->_m)))
#endif /* OFFSETOF */



/******************************************************************************/
/* Simple macros. */
/******************************************************************************/

#define IS_ETH_BROADCAST(_pEthAddr)                                         \
    (((unsigned char *) (_pEthAddr))[0] == ((unsigned char) 0xff))

#define IS_ETH_MULTICAST(_pEthAddr)                                         \
    (((unsigned char *) (_pEthAddr))[0] & ((unsigned char) 0x01))




/******************************************************************************/
/* Constants. */
/******************************************************************************/

#define ETHERNET_ADDRESS_SIZE           6
#define ETHERNET_PACKET_HEADER_SIZE     14
#define MIN_ETHERNET_PACKET_SIZE        64      /* with 4 byte crc. */
#define MAX_ETHERNET_PACKET_SIZE        1518    /* with 4 byte crc. */
#define MIN_ETHERNET_PACKET_SIZE_NO_CRC 60
#define MAX_ETHERNET_PACKET_SIZE_NO_CRC 1514
#define MAX_ETHERNET_PACKET_BUFFER_SIZE 1536    /* A nice even number. */

#ifndef LM_MAX_MC_TABLE_SIZE
#define LM_MAX_MC_TABLE_SIZE            32
#endif /* LM_MAX_MC_TABLE_SIZE */
#define LM_MC_ENTRY_SIZE                (ETHERNET_ADDRESS_SIZE+1)
#define LM_MC_INSTANCE_COUNT_INDEX      (LM_MC_ENTRY_SIZE-1)


/* Receive filter masks. */
#define LM_ACCEPT_UNICAST               0x0001
#define LM_ACCEPT_MULTICAST             0x0002
#define LM_ACCEPT_ALL_MULTICAST         0x0004
#define LM_ACCEPT_BROADCAST             0x0008
#define LM_ACCEPT_ERROR_PACKET          0x0010

#define LM_PROMISCUOUS_MODE             0x10000



/******************************************************************************/
/* PCI registers. */
/******************************************************************************/

#define PCI_VENDOR_ID_REG               0x00
#define PCI_DEVICE_ID_REG               0x02

#define PCI_COMMAND_REG                 0x04
#define PCI_IO_SPACE_ENABLE             0x0001
#define PCI_MEM_SPACE_ENABLE            0x0002
#define PCI_BUSMASTER_ENABLE            0x0004
#define PCI_MEMORY_WRITE_INVALIDATE     0x0010
#define PCI_PARITY_ERROR_ENABLE         0x0040
#define PCI_SYSTEM_ERROR_ENABLE         0x0100
#define PCI_FAST_BACK_TO_BACK_ENABLE    0x0200

#define PCI_STATUS_REG                  0x06
#define PCI_REV_ID_REG                  0x08

#define PCI_CACHE_LINE_SIZE_REG         0x0c

#define PCI_IO_BASE_ADDR_REG            0x10
#define PCI_IO_BASE_ADDR_MASK           0xfffffff0

#define PCI_MEM_BASE_ADDR_LOW           0x10
#define PCI_MEM_BASE_ADDR_HIGH          0x14

#define PCI_SUBSYSTEM_VENDOR_ID_REG     0x2c
#define PCI_SUBSYSTEM_ID_REG            0x2e
#define PCI_INT_LINE_REG                0x3c

#define PCIX_CAP_REG                    0x40
#define PCIX_ENABLE_RELAXED_ORDERING    BIT_17

/******************************************************************************/
/* Fragment structure. */
/******************************************************************************/

struct bm_fragment {
    u32 FragSize;
    u32 FragBuf;
};

struct bm_fraglist {
    /* FragCount is initialized for the caller to the maximum array size, on */
    /* return FragCount is the number of the actual fragments in the array. */
    u32 FragCount;

    /* Total buffer size. */
    u32 TotalSize;

    /* Fragment array buffer. */
    struct bm_fragment Fragments[1];
};

/******************************************************************************/
/* Status codes. */
/******************************************************************************/

#define LM_STATUS_SUCCESS                                       0
#define LM_STATUS_FAILURE                                       1

#define LM_STATUS_INTERRUPT_ACTIVE                              2
#define LM_STATUS_INTERRUPT_NOT_ACTIVE                          3

#define LM_STATUS_LINK_ACTIVE                                   4
#define LM_STATUS_LINK_DOWN                                     5
#define LM_STATUS_LINK_SETTING_MISMATCH                         6

#define LM_STATUS_TOO_MANY_FRAGMENTS                            7
#define LM_STATUS_TRANSMIT_ABORTED                              8
#define LM_STATUS_TRANSMIT_ERROR                                9
#define LM_STATUS_RECEIVE_ABORTED                               10
#define LM_STATUS_RECEIVE_ERROR                                 11
#define LM_STATUS_INVALID_PACKET_SIZE                           12
#define LM_STATUS_OUT_OF_MAP_REGISTERS                          13
#define LM_STATUS_UNKNOWN_ADAPTER                               14


/******************************************************************************/
/* Line speed. */
/******************************************************************************/

#define LM_LINE_SPEED_UNKNOWN                                   0
#define LM_LINE_SPEED_AUTO                  LM_LINE_SPEED_UNKNOWN
#define LM_LINE_SPEED_10MBPS                                    10
#define LM_LINE_SPEED_100MBPS                                   100


/******************************************************************************/
/* Duplex mode. */
/******************************************************************************/

#define LM_DUPLEX_MODE_UNKNOWN                                  0
#define LM_DUPLEX_MODE_HALF                                     1
#define LM_DUPLEX_MODE_FULL                                     2


/******************************************************************************/
/* Power state. */
/******************************************************************************/

#define LM_POWER_STATE_D0       0
#define LM_POWER_STATE_D1       1
#define LM_POWER_STATE_D2       2
#define LM_POWER_STATE_D3       3


/******************************************************************************/
/* Task offloading. */
/******************************************************************************/

#define LM_TASK_OFFLOAD_NONE                    0x0000
#define LM_TASK_OFFLOAD_TX_IP_CHECKSUM          0x0001
#define LM_TASK_OFFLOAD_RX_IP_CHECKSUM          0x0002
#define LM_TASK_OFFLOAD_TX_TCP_CHECKSUM         0x0004
#define LM_TASK_OFFLOAD_RX_TCP_CHECKSUM         0x0008
#define LM_TASK_OFFLOAD_TX_UDP_CHECKSUM         0x0010
#define LM_TASK_OFFLOAD_RX_UDP_CHECKSUM         0x0020
#define LM_TASK_OFFLOAD_TCP_SEGMENTATION        0x0040


/******************************************************************************/
/* Flow control. */
/******************************************************************************/

#define LM_FLOW_CONTROL_NONE                    0x00
#define LM_FLOW_CONTROL_RECEIVE_PAUSE           0x01
#define LM_FLOW_CONTROL_TRANSMIT_PAUSE          0x02
#define LM_FLOW_CONTROL_RX_TX_PAUSE (LM_FLOW_CONTROL_RECEIVE_PAUSE | \
    LM_FLOW_CONTROL_TRANSMIT_PAUSE)

/* This value can be or-ed with RECEIVE_PAUSE and TRANSMIT_PAUSE.  If the */
/* auto-negotiation is disabled and the RECEIVE_PAUSE and TRANSMIT_PAUSE */
/* bits are set, then flow control is enabled regardless of link partner's */
/* flow control capability. */
#define LM_FLOW_CONTROL_AUTO_PAUSE              0x80000000


/******************************************************************************/
/* Wake up mode. */
/******************************************************************************/

#define LM_WAKE_UP_MODE_NONE                    0
#define LM_WAKE_UP_MODE_MAGIC_PACKET            1
#define LM_WAKE_UP_MODE_NWUF                    2
#define LM_WAKE_UP_MODE_LINK_CHANGE             4

/******************************************************************************/
/* Counters. */
/******************************************************************************/

#define LM_COUNTER_FRAMES_XMITTED_OK                            0
#define LM_COUNTER_FRAMES_RECEIVED_OK                           1
#define LM_COUNTER_ERRORED_TRANSMIT_COUNT                       2
#define LM_COUNTER_ERRORED_RECEIVE_COUNT                        3
#define LM_COUNTER_RCV_CRC_ERROR                                4
#define LM_COUNTER_ALIGNMENT_ERROR                              5
#define LM_COUNTER_SINGLE_COLLISION_FRAMES                      6
#define LM_COUNTER_MULTIPLE_COLLISION_FRAMES                    7
#define LM_COUNTER_FRAMES_DEFERRED                              8
#define LM_COUNTER_MAX_COLLISIONS                               9
#define LM_COUNTER_RCV_OVERRUN                                  10
#define LM_COUNTER_XMIT_UNDERRUN                                11
#define LM_COUNTER_UNICAST_FRAMES_XMIT                          12
#define LM_COUNTER_MULTICAST_FRAMES_XMIT                        13
#define LM_COUNTER_BROADCAST_FRAMES_XMIT                        14
#define LM_COUNTER_UNICAST_FRAMES_RCV                           15
#define LM_COUNTER_MULTICAST_FRAMES_RCV                         16
#define LM_COUNTER_BROADCAST_FRAMES_RCV                         17

/******************************************************************************/
/* Forward definition. */
/******************************************************************************/

struct bm4400_hw;
struct bm4400_packet;

/******************************************************************************/
/* Function prototypes. */
/******************************************************************************/

int b44_LM_GetAdapterInfo(struct bm4400_hw * bm_dev);
int b44_LM_InitializeAdapter(struct bm4400_hw * bm_dev);
int b44_LM_ResetAdapter(struct bm4400_hw * bm_dev, char full);
int b44_LM_DisableInterrupt(struct bm4400_hw * bm_dev);
int b44_LM_EnableInterrupt(struct bm4400_hw * bm_dev);
int b44_LM_SendPacket(struct bm4400_hw * bm_dev, struct bm4400_packet * packet);
int b44_LM_ServiceInterrupts(struct bm4400_hw * bm_dev);
int b44_LM_QueueRxPackets(struct bm4400_hw * bm_dev);
int b44_LM_SetReceiveMask(struct bm4400_hw * bm_dev, u32 Mask);
int b44_LM_Halt(struct bm4400_hw * bm_dev);
int b44_LM_Abort(struct bm4400_hw * bm_dev);
int b44_LM_MulticastAdd(struct bm4400_hw * bm_dev, u8 * pMcAddress);
int b44_LM_MulticastDel(struct bm4400_hw * bm_dev, u8 * pMcAddress);
int b44_LM_MulticastClear(struct bm4400_hw * bm_dev);
int b44_LM_SetMacAddress(struct bm4400_hw * bm_dev, u8 * pMacAddress);
int b44_LM_LoopbackAddress(struct bm4400_hw * bm_dev, u8 * pAddress);

u32 b44_LM_PMCapabilities(struct bm4400_hw * bm_dev);
int b44_LM_SetPowerState(struct bm4400_hw * bm_dev, u32 PowerLevel);

void b44_LM_ReadPhy(struct bm4400_hw * bm_dev, u32 PhyReg,
    u32 *pData32);
void b44_LM_WritePhy(struct bm4400_hw * bm_dev, u32 PhyReg,
    u32 Data32);

int b44_LM_SetupPhy(struct bm4400_hw * bm_dev);
int b44_LM_ResetPhy(struct bm4400_hw * bm_dev);
int b44_LM_GetStats(struct bm4400_hw * bm_dev);
int b44_LM_NvramRead(struct bm4400_hw * bm_dev, u32 offset,
    u32 *pData);
int b44_LM_NvramWriteBlock(struct bm4400_hw * bm_dev, u32 offset,
    u32 *pData, u32 Size);
void b44_LM_PollLink(struct bm4400_hw * bm_dev);
int b44_LM_StatsUpdate(struct bm4400_hw * bm_dev);
#ifdef BCM_WOL
void b44_LM_pmset(struct bm4400_hw * bm_dev);
#endif

/******************************************************************************/
/* These are the OS specific functions called by LMAC. */
/******************************************************************************/

int b44_MM_ReadConfig16(struct bm4400_hw * bm_dev, u32 offset,
    u16 *pValue16);
int b44_MM_WriteConfig16(struct bm4400_hw * bm_dev, u32 offset,
    u16 Value16);
int b44_MM_ReadConfig32(struct bm4400_hw * bm_dev, u32 offset,
    u32 *pval);
int b44_MM_WriteConfig32(struct bm4400_hw * bm_dev, u32 offset,
    u32 val);
int b44_MM_MapMemBase(struct bm4400_hw * bm_dev);
int b44_MM_MapIoBase(struct bm4400_hw * bm_dev);
int b44_MM_IndicateRxPackets(struct bm4400_hw * bm_dev);
int b44_MM_IndicateTxPackets(struct bm4400_hw * bm_dev);
int b44_MM_StartTxDma(struct bm4400_hw * bm_dev, struct bm4400_packet * packet);
int b44_MM_CompleteTxDma(struct bm4400_hw * bm_dev, struct bm4400_packet * packet);
int b44_MM_AllocateMemory(struct bm4400_hw * bm_dev, u32 BlockSize, 
    void * *virt);
int b44_MM_AllocateSharedMemory(struct bm4400_hw * bm_dev, u32 BlockSize,
    void * *virt, u32 *phys);
int b44_MM_GetConfig(struct bm4400_hw * bm_dev);
int b44_MM_IndicateStatus(struct bm4400_hw * bm_dev, int Status);
int b44_MM_InitializeUmPackets(struct bm4400_hw * bm_dev);
int b44_MM_FreeRxBuffer(struct bm4400_hw * bm_dev, struct bm4400_packet * packet);


#endif /* B44LM_H */

