/******************************************************************************/
/*                                                                            */
/* Broadcom BCM4400 Linux Network Driver, Copyright (c) 2002 Broadcom         */
/* Corporation.                                                               */
/* All rights reserved.                                                       */
/*                                                                            */
/* This program is free software; you can redistribute it and/or modify       */
/* it under the terms of the GNU General Public License as published by       */
/* the Free Software Foundation, located in the file LICENSE.                 */
/*                                                                            */
/* Queue functions.                                                           */
/*    void          QQ_InitQueue(struct bm_queue * queue)                     */
/*    char          QQ_Full(struct bm_queue * queue)                          */
/*    char          QQ_Empty(struct bm_queue * queue)                         */
/*    unsigned int  QQ_GetSize(struct bm_queue * queue)                       */
/*    unsigned int  QQ_GetEntryCnt(struct bm_queue * queue)                   */
/*    char          QQ_PushHead(struct bm_queue * queue, void * entry)        */
/*    char          QQ_PushTail(struct bm_queue * queue, void * entry)        */
/*    void *        QQ_PopHead(struct bm_queue * queue)                       */
/*    void *        QQ_PopTail(struct bm_queue * queue)                       */
/*    void *        QQ_GetHead(struct bm_queue * queue, unsigned int i)       */
/*    void *        QQ_GetTail(struct bm_queue * queue, unsigned int i)       */
/*                                                                            */
/*                                                                            */
/* History:                                                                   */
/*    2000/02/25 Hav Khauv        Initial version.                            */
/*    2003/06/14 Alan Cox         Extensively tided into Linux coding style   */
/* 				  Fixed failure to work on boxes with "char"  */
/*                                types unsigned			      */
/******************************************************************************/

#ifndef BCM_QUEUE_H
#define BCM_QUEUE_H



/******************************************************************************/
/* Queue definitions. */
/******************************************************************************/

/* Entry for queueing. */

/* Queue header -- base type. */

struct bm_queue {
	unsigned int head;
	unsigned int tail;
	unsigned int size;
	atomic_t entries;
	void *array[1];
};

/* Declare queue type macro. */
#define DECLARE_QUEUE_TYPE(_QUEUE_TYPE, _QUEUE_SIZE)            \
                                                                \
    typedef struct {                                            \
        struct bm_queue Container;                                 \
        void *EntryBuffer[_QUEUE_SIZE];                     \
    } _QUEUE_TYPE, *P##_QUEUE_TYPE



/******************************************************************************/
/* Compilation switches. */
/******************************************************************************/

#if DBG
#undef QQ_NO_OVERFLOW_CHECK
#undef QQ_NO_UNDERFLOW_CHECK
#endif				/* DBG */

#ifdef QQ_USE_MACROS
/* notdone */
#else

#ifdef QQ_NO_INLINE
#define static inline
#endif				/* QQ_NO_INLINE */


/**
 *	QQ_InitQueue	-	Set up a ring buffer queue
 *	@queue: Queue to set up
 *	@size: Size of Queue
 *
 *	Initialise a ring buffer queue read for use
 */
 
static inline void QQ_InitQueue(struct bm_queue *queue, unsigned int size)
{
	queue->head = 0;
	queue->tail = 0;
	queue->size = size + 1;
	atomic_set(&queue->entries, 0);
}		

/**
 *	QQ_Full		-	Test if a ring buffer queue is full
 *	@queue: Queue to test
 *
 *	Returns true if the queue is full, otherwise false
 */
 
static inline int QQ_Full(struct bm_queue *queue)
{
	unsigned int next  = (queue->head + 1) % queue->size;
	return (next == queue->tail);
}

/**
 *	QQ_Empty		-	Test if a ring buffer queue is full
 *	@queue: Queue to test
 *
 *	Returns true if the queue is empty, otherwise false
 */
 
static inline char QQ_Empty(struct bm_queue *queue)
{
	return (queue->head == queue->tail);
}

/**
 *	QQ_GetSize		-	Return the size of the queue
 *	@queue: Queue to check
 *
 *	Returns the size (limit size not current entries) of the
 *	queue.
 */
 
static inline unsigned int QQ_GetSize(struct bm_queue *queue)
{
	return queue->size;
}

/**
 *	QQ_GetEntryCnt		-	Item count
 *	@queue: Queue to check
 *
 *	Return the number of items currently in the ring buffer
 *	queue.
 */
 
static inline unsigned int QQ_GetEntryCnt(struct bm_queue *queue)
{
	return atomic_read(&queue->entries);
}

/**
 *	QQ_PushHead	-	Push an entry onto the queue start
 *	@queue: Queue to use
 *	@entry: Opaque pointer to data
 *
 *	Push a data item onto the front of the queue. Returns false
 *	if the queue is already full.
 */
 
static inline int QQ_PushHead(struct bm_queue *queue, void *entry)
{
	unsigned int head;

	head = (queue->head + 1) % queue->size;

#if !defined(QQ_NO_OVERFLOW_CHECK)
	if (head == queue->tail)
		return 0;
#endif				/* QQ_NO_OVERFLOW_CHECK */

	queue->array[queue->head] = entry;
	wmb();
	queue->head = head;
	atomic_inc(&queue->entries);

	return 1;
}

/**
 *	QQ_PushTail	-	Push an entry onto the queue end
 *	@queue: Queue to use
 *	@entry: Opaque pointer to data
 *
 *	Push a data item onto the end of the queue. Returns false
 *	if the queue is already full.
 */
 
static inline char QQ_PushTail(struct bm_queue *queue, void *entry)
{
	unsigned int tail;

	tail = queue->tail;
	if (tail == 0) {
		tail = queue->size;
	}			/* if */
	tail--;

#if !defined(QQ_NO_OVERFLOW_CHECK)
	if (tail == queue->head)
		return 0;
#endif				/* QQ_NO_OVERFLOW_CHECK */

	queue->array[tail] = entry;
	wmb();
	queue->tail = tail;
	atomic_inc(&queue->entries);

	return -1;
}				/* QQ_PushTail */

/**
 *	QQ_PopHead	-	Pull an entry off the queue start
 *	@queue: Queue to use
 *
 *	Pull a data item off the start of the queue. Returns NULL
 *	if the queue is already empty. Returns the opaque data 
 *	pointer that was pushed if not .
 */
 
static inline void *QQ_PopHead(struct bm_queue *queue)
{
	unsigned int head;
	void *entry;

	head = queue->head;

#if !defined(QQ_NO_UNDERFLOW_CHECK)
	if (head == queue->tail)
		return NULL;
#endif				/* QQ_NO_UNDERFLOW_CHECK */

	if (head == 0)
		head = queue->size;

	head--;

	entry = queue->array[head];
	mb();
	queue->head = head;
	atomic_dec(&queue->entries);

	return entry;
}				/* QQ_PopHead */


/**
 *	QQ_PopTail	-	Pull an entry off the queue end
 *	@queue: Queue to use
 *
 *	Pull a data item off the end of the queue. Returns NULL
 *	if the queue is already empty. Returns the opaque data 
 *	pointer that was pushed if not.
 */
 
static inline void *QQ_PopTail(struct bm_queue *queue)
{
	unsigned int tail;
	void *entry;

	tail = queue->tail;

#if !defined(QQ_NO_UNDERFLOW_CHECK)
	if (tail == queue->head)
		return NULL;
#endif				/* QQ_NO_UNDERFLOW_CHECK */

	entry = queue->array[tail];
	mb();
	queue->tail = (tail + 1) % queue->size;
	atomic_dec(&queue->entries);

	return entry;
}				/* QQ_PopTail */

/**
 *	QQ_GetHead	-	Peek at the entry on the queue start
 *	@queue: Queue to use
 *
 *	Look at the data item at the start of the queue. Returns NULL
 *	if the queue is already empty. Returns the opaque data 
 *	pointer that was pushed if not
 */

static inline void *QQ_GetHead(struct bm_queue *queue, unsigned int i)
{
	if (i >= atomic_read(&queue->entries))
		return NULL;

	if (queue->head > i)
		i = queue->head - i;
	else
		i = queue->size - (i - queue->head);
	i--;

	return queue->array[i];
}

/**
 *	QQ_GetTail	-	Peek at the entry on the queue end
 *	@queue: Queue to use
 *
 *	Look at the data item at the end of the queue. Returns NULL
 *	if the queue is already empty. Returns the opaque data 
 *	pointer that was pushed if not
 */

static inline void *QQ_GetTail(struct bm_queue *queue, unsigned int i)
{
	if (i >= atomic_read(&queue->entries))
		return NULL;

	i += queue->tail;
	if (i >= queue->size)
		i = i - queue->size;

	return queue->array[i];
}

#endif				/* QQ_USE_MACROS */



#endif				/* QUEUE_H */
